"""NBA games table"""

from .dynamodb_table import DynamoDBTable

class NbaTeamsTable(DynamoDBTable):
    def __init__(self, environment):
        DynamoDBTable.__init__(self, 'nba-teams', environment, 'teamId')
